@@ -16,7 +16,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2004 Duke University
 
-# $Id: dumpMetadata.py,v 1.48 2007/05/18 15:01:16 pnasrat Exp $
+# $Id$
 
 import os
 import rpm
@@ -28,7 +28,10 @@ import struct
 import re
 import stat
 import bz2
-import sqlitecachec
+try:
+    import sqlitecachec
+except ImportError:
+    pass
 
 # done to fix gzip randomly changing the checksum
 import gzip
@@ -217,9 +220,12 @@ def byteranges(file):
     
 
 class MDError(exceptions.Exception):
-    def __init__(self, args=None):
+    def __init__(self, value=None):
         exceptions.Exception.__init__(self)
-        self.args = args
+        self.value = value
+    
+    def __str__(self):
+        return self.value
 
 
 
@@ -596,7 +602,9 @@ class RpmMetaData:
 
         key = md5.new("".join(t)).hexdigest()
                                         
-        csumtag = '%s-%s' % (self.hdr['name'] , key)
+        csumtag = '%s-%s-%s-%s' % (os.path.basename(self.relativepath), 
+                                   self.hdr[rpm.RPMTAG_SHA1HEADER], 
+                                   self.size, self.mtime)
         csumfile = '%s/%s' % (self.options['cachedir'], csumtag)
         if os.path.exists(csumfile) and self.mtime <= os.stat(csumfile)[8]:
             csumo = open(csumfile, 'r')
